@@ -35,7 +35,7 @@ DEFAULT_DOCKER_ENV_FILE_CONTENT = dict(
     POSTGRES_PASSWORD="changeme",
     HASURA_GRAPHQL_DATABASE_URL="postgres://dipdup:changeme@db:5432/dipdup",
     HASURA_GRAPHQL_ENABLE_CONSOLE="true",
-    HASURA_GRAPHQL_DEV_MODE="true",
+    HASURA_GRAPHQL_ADMIN_INTERNAL_ERRORS="true",
     HASURA_GRAPHQL_ENABLED_LOG_TYPES="startup, http-log, webhook-log, websocket-log, query-log",
     HASURA_GRAPHQL_ADMIN_SECRET="changeme",
     HASURA_GRAPHQL_UNAUTHORIZED_ROLE="user",
