@@ -71,9 +71,9 @@ class PostgresDatabaseConfig:
 
     kind: Literal['postgres']
     host: str
-    port: int
-    user: str
-    database: str
+    user: str = 'postgres'
+    database: str = 'postgres'
+    port: int = 5432
     schema_name: str = 'public'
     password: str = ''
     immune_tables: Optional[List[str]] = None
@@ -321,6 +321,7 @@ class ParameterTypeMixin:
 
     def initialize_parameter_cls(self, package: str, module_name: str, entrypoint: str) -> None:
         _logger.info('Registering parameter type for entrypoint `%s`', entrypoint)
+        entrypoint = entrypoint.lstrip('_')
         module_name = f'{package}.types.{module_name}.parameter.{pascal_to_snake(entrypoint)}'
         cls_name = snake_to_pascal(entrypoint) + 'Parameter'
         self.parameter_type_cls = import_from(module_name, cls_name)
@@ -661,7 +662,6 @@ class HasuraConfig:
     camel_case: bool = False
     connection_timeout: int = 5
     rest: bool = True
-    reset: Optional[bool] = None
     http: Optional[HTTPConfig] = None
 
     @validator('url', allow_reuse=True)
